@@ -1,10 +1,11 @@
 /* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only */
-/* Copyright (c) 2017-2023 Brett Sheffield <bacs@librecast.net> */
+/* Copyright (c) 2017-2024 Brett Sheffield <bacs@librecast.net> */
 
 #ifndef _LIBRECAST_TYPES_H
 #define _LIBRECAST_TYPES_H 1
 
 #include <librecast/errors.h>
+#include <librecast/crypto.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -14,6 +15,16 @@
 # include <lcrq.h>
 #endif
 
+#ifndef ENOLINK
+# define ENOLINK         95	/* Link has been severed */
+#endif
+#ifndef ENOKEY
+# define ENOKEY          126	/* Required key not available */
+#endif
+#ifndef UIO_MAXIOV
+# define UIO_MAXIOV      1024
+#endif
+
 #define LC_DEFAULT_PORT 4242
 #define LC_DEFAULT_FLAGS 0x1e
 #define LC_BRIDGE_NAME "lc0"
@@ -22,6 +33,10 @@
 #define DEFAULT_MULTICAST_HOPS 255
 #define LC_DEFAULT_RQ_T 1024 /* default symbol size for RaptorQ */
 
+/* bloom filter params */
+#define BLOOM_LEN 16777216 /* 16MiB */
+#define BLOOM_HASHES 8
+
 /* define flag to drop packets when used with send* calls
  * used for testing */
 #define MSG_DROP MSG_PEEK
@@ -35,6 +50,7 @@ typedef struct lc_channel_t lc_channel_t;
 typedef struct lc_msg_head_t lc_msg_head_t;
 typedef struct lc_query_t lc_query_t;
 typedef struct lc_query_param_t lc_query_param_t;
+typedef struct lc_router_s lc_router_t;
 typedef struct lc_stat_s lc_stat_t;
 typedef struct lc_sync_options_s lc_sync_options_t;
 typedef struct mdex_s mdex_t;
@@ -97,6 +113,17 @@ typedef enum {
 	LC_ATTR_OPCODE,
 } lc_msg_attr_t;
 
+typedef enum {
+        LC_TLV_NOP =  0x00,
+        LC_TLV_DST =  0x01,
+        LC_TLV_SRC =  0x02,
+        LC_TLV_HASH = 0x04,
+        LC_TLV_JOIN = 0x08,
+        LC_TLV_PART = 0x10,
+        LC_TLV_STOP = 0x20,
+        LC_TLV_PEER = 0x40,
+} lc_tlv_type_t;
+
 /* channel encoding constants */
 typedef enum {
 	LC_CODE_NONE           = 0x00,
@@ -116,11 +143,30 @@ enum {
         LCTX_DEBUG_SYNCFILE = 0x00000001, /* print files synced to lctx->stream */
 };
 
+typedef enum {
+        LC_SOCK_IN6   = 0, /* IPv6 multicast */
+        LC_SOCK_PAIR  = 1, /* connected socket pair */
+        LC_SOCK_WSS   = 2, /* Websocket (RFC 6455) */
+        LC_SOCK_RTC   = 3, /* WebRTC */
+} lc_socktype_t;
+
 typedef struct lc_key_s {
 	unsigned char *key;
 	size_t keylen;
 } lc_key_t;
 
+typedef struct lc_tlv_s {
+        uint8_t  type;
+        uint32_t len;
+        uint8_t  value[];
+} __attribute__((__packed__)) lc_tlv_t;
+
+typedef struct lc_tlv_hash_s {
+        uint8_t  type;
+        uint32_t len;
+        uint8_t  value[HASHSIZE];
+} __attribute__((__packed__)) lc_tlv_hash_t;
+
 typedef struct lc_message_t {
 	uint64_t timestamp;
 	struct in6_addr dst;
