@@ -7,13 +7,17 @@
 #include "job.h"
 #include "log.h"
 
+#define aload(x) __atomic_load_n((x), __ATOMIC_ACQUIRE)
+#define astor(x, y) __atomic_store_n((x),(y), __ATOMIC_RELEASE)
+
 job_t *job_shift(job_queue_t *q)
 {
 	job_t *job;
 	if (sem_wait(&q->lock) == -1) return NULL;
-	if ((job = q->next)) {
-		q->next = q->next->next;
-		if (!q->next) q->last = NULL;
+	if ((job = aload(&q->next))) {
+		job_t *next = aload(&job->next);
+		astor(&q->next, next);
+		if (!next) astor(&q->last, NULL);
 	}
 	sem_post(&q->lock);
 	return job;
