@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only */
-/* Copyright (c) 2020-2023 Brett Sheffield <bacs@librecast.net> */
+/* Copyright (c) 2020-2024 Brett Sheffield <bacs@librecast.net> */
 
 #define _GNU_SOURCE /* required for struct in6_pktinfo */
 #include "config.h"
@@ -461,8 +461,10 @@ static void *mld_state_grp_add(void *arg)
 
 void mld_watch_del(mld_watch_t *watch)
 {
-	if (watch->prev) watch->prev->next = watch->next;
-	else watch->mld->watch = watch->next;
+	mld_watch_t *next = aload(&watch->next);
+	mld_watch_t *prev = aload(&watch->prev);
+	if (prev) astor(&prev->next, next);
+	else astor(&watch->mld->watch, next);
 	free(watch);
 }
 
@@ -483,7 +485,7 @@ mld_watch_t *mld_watch_add(mld_t *mld, unsigned int ifx, struct in6_addr *addr,
 		watch->next = mld->watch;
 		mld->watch->prev = watch;
 	}
-	mld->watch = watch;
+	astor(&mld->watch, watch);
 
 	return watch;
 }
@@ -563,7 +565,7 @@ static void *mld_watch_event(void *arg)
 	if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED))
 		goto err_attr_destroy;
 	txn = (mld_txn_t *)arg;
-	for (mld_watch_t *w = txn->mld->watch; w; w = w->next) {
+	for (mld_watch_t *w = aload(&txn->mld->watch); w; w = aload(&w->next)) {
 		if (w->ifx && txn->ifx && txn->ifx != w->ifx) continue;
 		if (w->grp.s6_addr[0] && txn->addr && memcmp(txn->addr, &w->grp, sizeof(struct in6_addr)))
 			continue;
@@ -1087,7 +1089,7 @@ static int mld_listen(mld_t *mld)
 		goto err_close_sock;
 	}
 
-	mld->listen_sock = sock;
+	astor(&mld->listen_sock, sock);
 	/* prepare scatter/gather structs */
 	iov[0].iov_base = &ethh;
 	iov[0].iov_len = sizeof ethh;
@@ -1101,6 +1103,7 @@ static int mld_listen(mld_t *mld)
 	msg.msg_iovlen = sizeof iov / sizeof iov[0];
 	msg.msg_flags = 0;
 	while (1) {
+		sock = aload(&mld->listen_sock);
 		byt = recvmsg(sock, &msg, 0);
 		if (byt == -1) {
 			ERROR("read/recvmsg");
@@ -1112,7 +1115,6 @@ static int mld_listen(mld_t *mld)
 		DEBUG("received type %u on %u (%zi bytes)\n", mldh->mld_icmp6_hdr.icmp6_type, ifx, byt);
 		if (ifx) mld_msg_handler(mld, ifx, ip6h, mldh);
 	}
-	rc = 0;
 err_close_sock:
 	close(sock);
 	return rc;
@@ -1325,10 +1327,12 @@ err_free_mld:
 
 void mld_stop(mld_t *mld)
 {
+	for (int i = 0; i < MLD_THREADS; i++) if (mld->q[i]) job_queue_destroy(mld->q[i]);
 #ifdef __linux__
-	if (mld->listen_sock >= 0) {
-		close(mld->listen_sock);
-		mld->listen_sock = -1;
+	int sock = aload(&mld->listen_sock);
+	if (sock >= 0) {
+		close(sock);
+		astor(&mld->listen_sock, -1);
 	}
 #else
 	for (mld_iface_t *iface = mld->iface; iface; iface = iface->next) {
@@ -1338,7 +1342,6 @@ void mld_stop(mld_t *mld)
 		}
 	}
 #endif
-	for (int i = 0; i < MLD_THREADS; i++) if (mld->q[i]) job_queue_destroy(mld->q[i]);
 }
 
 mld_t *mld_start(mld_t *mld)
