@@ -3,7 +3,11 @@
 
 #ifndef _TEST_H
 #define _TEST_H 1
+#ifdef __linux__
+# define _GNU_SOURCE
+#endif
 #include "../src/config.h"
+#include <libgen.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -32,6 +36,7 @@ extern int test_status;
 void test_init(void);
 void fail_msg(char *msg, ...);
 int test_assert(int condition, char *msg, ...);
+int test_assert_q(int condition, char *msg, ...);
 int test_strcmp(char *str1, char *str2, char *msg, ...);
 int test_strncmp(char *str1, char *str2, size_t len, char *msg, ...);
 int test_expect(char *expected, char *got);
