@@ -1,6 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only */
-/* Copyright (c) 2023 Brett Sheffield <bacs@librecast.net> */
+/* Copyright (c) 2023-2024 Brett Sheffield <bacs@librecast.net> */
 
+#define _GNU_SOURCE
 #include <librecast_pvt.h>
 #include <librecast/crypto.h>
 #include <librecast/net.h>
@@ -36,6 +37,7 @@ inline static void _lc_channel_part(lc_channel_t *chan)
 }
 
 /* return number of bits set in bitmap (Hamming Weight) */
+#ifdef HAVE_RQ_OTI
 static unsigned int hamm(unsigned char *map, size_t len)
 {
 	unsigned int c = 0;
@@ -330,9 +332,18 @@ static ssize_t share_send_data(lc_share_t *share, struct in6_addr *grp, struct m
 	lc_channel_coding_set(chan, share->lctx->coding | coding);
 	rc = lc_channel_send(chan, mem->data, mem->size, 0);
 	if (rc == -1 || !chan->rq) goto err_channel_free;
+	if (share->stats) {
+		share->stats->byt_gross += rc;
+		share->stats->byt_info += mem->size;
+		share->stats->pkts_io++;
+	}
 	for (int i = 1; i < rq_KP(chan->rq) + RQ_OVERHEAD * 2; i++) {
 		rc = lc_channel_send(chan, NULL, 0, 0);
 		if (rc == -1) break;
+		if (share->stats) {
+			share->stats->byt_gross += rc;
+			share->stats->pkts_io++;
+		}
 	}
 err_channel_free:
 	lc_channel_free(chan);
@@ -340,6 +351,7 @@ err_socket_close:
 	lc_socket_close(sock);
 	return rc;
 }
+#endif /* HAVE_MLD */
 
 static char *concat_mdex_root_file(const char *root, const char *file)
 {
@@ -513,7 +525,6 @@ static void share_watch_callback(mld_watch_t *watch)
 		}
 	}
 }
-#endif
 
 lc_share_t * lc_share(lc_ctx_t *lctx, mdex_t *mdex, unsigned int ifx, lc_stat_t *stats,
 		lc_sync_options_t *opt, int flags)
@@ -530,6 +541,7 @@ lc_share_t * lc_share(lc_ctx_t *lctx, mdex_t *mdex, unsigned int ifx, lc_stat_t
 	share->tsend = calloc(nthread_send, sizeof(pthread_t));
 	if (!share->tsend) { err = errno; goto err_free_share; }
 	share->lctx = lctx;
+	share->stats = stats;
 	share->mdex = mdex;
 	share->ifx = ifx;
 	share->flags = flags;
@@ -560,7 +572,7 @@ err_free_share:
 #else
 	(void)lctx; (void)mdex; (void)ifx; (void)stats; (void)opt; (void)flags; /* unused */
 	return (errno = ENOTSUP), NULL;
-#endif
+#endif /* HAVE_MLD */
 }
 
 void lc_unshare(lc_share_t *share)
@@ -581,6 +593,7 @@ void lc_unshare(lc_share_t *share)
 	(void)share;
 #endif
 }
+#endif /* HAVE_LIBLCRQ */
 
 int lc_memsync(void *dst, void *src, const size_t n, q_t *q, lc_stat_t *stats,
 		lc_sync_options_t *opt, int flags)
@@ -687,6 +700,7 @@ static int setfilestat(const char *pathname, struct stat *sb, int flags)
 	return rc;
 }
 
+#ifdef HAVE_RQ_OTI
 static int setfilemeta(const char *pathname, net_tree_t *data, int flags)
 {
 	struct timespec *atime, *mtime;
@@ -701,6 +715,7 @@ static int setfilemeta(const char *pathname, net_tree_t *data, int flags)
 	if (atime || mtime) rc += utimeset(pathname, atime, mtime);
 	return rc;
 }
+#endif
 
 static int syncdirlocal(mdex_t *sdex, mdex_t *ddex, const char *src, const char *dst, size_t top, q_t *q,
 		lc_stat_t *stats, lc_sync_options_t *opt, int flags);
@@ -915,6 +930,7 @@ err_errno:
 	return rc;
 }
 
+#ifdef HAVE_RQ_OTI
 ssize_t lc_sync(lc_ctx_t *lctx, unsigned char *hash, void *data, const size_t len,
 		q_t *q, lc_stat_t *stats, lc_sync_options_t *opt, int flags)
 {
@@ -1292,3 +1308,4 @@ err_free_opt:
 	if (optalloc) free(opt);
 	return rc;
 }
+#endif /* HAVE_LIBLCRQ */
